# -*- coding: utf-8 -*-

from rest_framework import serializers
from thx.api.serializers import (
    ThuxCreateSerializerMixin, ThuxDisableSerializerMixin, ThuxEnableSerializerMixin, ThuxListSerializerMixin,
    ThuxPartialUpdateSerializerMixin, ThuxRetrieveSerializerMixin, ThuxUpdateSerializerMixin,
)
from thx.role.core.serializers import ThuxRoleSerializerMixin

from ... import models


# CARRIAGE
class CarriageSerializer(ThuxRoleSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_carriage_carriage'
        model = models.Carriage
        fields = '__all__'


class CarriageListSerializer(ThuxRoleSerializerMixin, ThuxListSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_carriage_carriage_list'
        model = models.Carriage
        fields = '__all__'


class CarriageRetrieveSerializer(ThuxRoleSerializerMixin, ThuxRetrieveSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_carriage_carriage_retrieve'
        model = models.Carriage
        fields = '__all__'


class CarriageCreateSerializer(ThuxRoleSerializerMixin, ThuxCreateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_carriage_carriage_create'
        model = models.Carriage
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = CarriageRetrieveSerializer


class CarriageUpdateSerializer(ThuxRoleSerializerMixin, ThuxUpdateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_carriage_carriage_update'
        model = models.Carriage
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = CarriageRetrieveSerializer


class CarriagePartialUpdateSerializer(
    ThuxRoleSerializerMixin, ThuxPartialUpdateSerializerMixin, serializers.ModelSerializer
):
    class Meta:
        ref_name = 'v1_carriage_carriage_partial_update'
        model = models.Carriage
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = CarriageRetrieveSerializer


class CarriageEnableSerializer(ThuxRoleSerializerMixin, ThuxEnableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_carriage_carriage_enable'
        model = models.Carriage
        fields = ()
        response_serializer = CarriageRetrieveSerializer

    def update(self, instance, validated_data):
        instance.status = 1
        instance.save()
        return instance


class CarriageDisableSerializer(ThuxRoleSerializerMixin, ThuxDisableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_carriage_carriage_disable'
        model = models.Carriage
        fields = ()
        response_serializer = CarriageRetrieveSerializer

    def update(self, instance, validated_data):
        instance.status = 0
        instance.save()
        return instance


class CarriageLightSerializer(ThuxRoleSerializerMixin, ThuxListSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_carriage_carriage_light'
        model = models.Carriage
        fields = ('code', 'status', 'status_display',)


# DELIVERY
class DeliverySerializer(ThuxRoleSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_carriage_delivery'
        model = models.Delivery
        fields = '__all__'


class DeliveryListSerializer(ThuxRoleSerializerMixin, ThuxListSerializerMixin, serializers.ModelSerializer):
    carriage = CarriageRetrieveSerializer()

    class Meta:
        ref_name = 'v1_carriage_delivery_list'
        model = models.Delivery
        fields = '__all__'


class DeliveryRetrieveSerializer(ThuxRoleSerializerMixin, ThuxRetrieveSerializerMixin, serializers.ModelSerializer):
    carriage = CarriageRetrieveSerializer()

    class Meta:
        ref_name = 'v1_carriage_delivery_retrieve'
        model = models.Delivery
        fields = '__all__'


class DeliveryCreateSerializer(ThuxRoleSerializerMixin, ThuxCreateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_carriage_delivery_create'
        model = models.Delivery
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = DeliveryRetrieveSerializer


class DeliveryUpdateSerializer(ThuxRoleSerializerMixin, ThuxUpdateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_carriage_delivery_update'
        model = models.Delivery
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = DeliveryRetrieveSerializer


class DeliveryPartialUpdateSerializer(
    ThuxRoleSerializerMixin, ThuxPartialUpdateSerializerMixin, serializers.ModelSerializer
):
    class Meta:
        ref_name = 'v1_carriage_delivery_partial_update'
        model = models.Delivery
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = DeliveryRetrieveSerializer


class DeliveryEnableSerializer(ThuxRoleSerializerMixin, ThuxEnableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_carriage_delivery_enable'
        model = models.Delivery
        fields = ()
        response_serializer = DeliveryRetrieveSerializer

    def update(self, instance, validated_data):
        instance.status = 1
        instance.save()
        return instance


class DeliveryDisableSerializer(ThuxRoleSerializerMixin, ThuxDisableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_carriage_delivery_disable'
        model = models.Delivery
        fields = ()
        response_serializer = DeliveryRetrieveSerializer

    def update(self, instance, validated_data):
        instance.status = 0
        instance.save()
        return instance


class DeliveryLightSerializer(ThuxRoleSerializerMixin, ThuxListSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_carriage_delivery_light'
        model = models.Delivery
        fields = ('code', 'status', 'status_display',)
